<?php


namespace MirfalahTech\Laravel\Payment\Gateway\DummyPay;


use Illuminate\Events\Dispatcher;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response as Res;
use MirfalahTech\Laravel\Payment\Contracts\PayableEntity;
use MirfalahTech\Laravel\Payment\Contracts\PaymentGatewayDriver;
use MirfalahTech\Laravel\Payment\Events\PaymentFailedEvent;
use MirfalahTech\Laravel\Payment\Events\PaymentPendingEvent;
use MirfalahTech\Laravel\Payment\Events\PaymentSuccessEvent;
use MirfalahTech\Laravel\Payment\Events\ReturnVerifiedEvent;
use MirfalahTech\Laravel\Payment\Facade\Payment;
use MirfalahTech\Laravel\Payment\Traits\BillStatusBoolean;
use Symfony\Component\HttpFoundation\Response;

class DummyPayDriver implements PaymentGatewayDriver
{
    use BillStatusBoolean;

    /**
     * @var string
     */
    protected $callback_url;

    /**
     * @var string
     */
    protected $return_url;

    /**
     * @var Dispatcher
     */
    protected $events;

    public function __construct(Dispatcher $events, string $callback_url, string $return_url){
        $this->callback_url = rtrim($callback_url, '/');
        $this->return_url = rtrim($return_url, '/');
        $this->events = $events;
    }

    public function createPaymentURL(PayableEntity $payable, array $options = []): string
    {
        $order_id = $payable->getBillId();
        return "$this->callback_url?".http_build_query(compact('order_id'));
    }

    public function verifyGatewayReturn(Request $request): bool
    {
        $this->events->dispatch(new ReturnVerifiedEvent('dummy', $this, $request));
        $this->dispatchStatusEvent($request);
        return true;
    }

    public function verifyGatewayCallback(Request $request, Response &$response = null): bool
    {
        $order_id = $request->query->get('order_id');
        if($request->query->has('status')){
            $response = Res::redirectTo($this->return_url.'?'.http_build_query(['order_id' => $order_id, 'status' => 'success']));
            return true;
        } else {
            /** @noinspection HtmlUnknownTarget */
            $html = sprintf(
                '<a href="%s">Click Here for Positive Response</a><br/><br/><a href="%s">Click Here for Negative Response</a>',
                $this->callback_url.'?'.http_build_query(['order_id' => $order_id, 'status' => 'success']),
                $this->callback_url.'?'.http_build_query(['order_id' => $order_id, 'status' => 'failed'])
            );
            $response = Res::make($html);
            return false;
        }
    }

    public function getBillIdFromRequest(Request $request): ?string{
        return $request->query->get('order_id');
    }

    public function getBillStatus(Request $request): ?int
    {
        return $request->query->get('status') == 'success' ?
            Payment::SUCCESS : Payment::FAILED;
    }

    protected function dispatchStatusEvent(Request $request)
    {
        switch ($this->getBillStatus($request)) {
            case Payment::SUCCESS:
                $this->events->dispatch(new PaymentSuccessEvent('dummy', $this, $request));
                break;
            case Payment::PENDING:
                $this->events->dispatch(new PaymentPendingEvent('dummy', $this, $request));
                break;
            case Payment::FAILED:
                $this->events->dispatch(new PaymentFailedEvent('dummy', $this, $request));
                break;
        }
    }
}